"""Command line tool for mkbook."""

from __future__ import annotations

import argparse
import shutil
import sys
import textwrap
from pathlib import Path

from . import MakeBook, __version__


class MBHelpFormatter(
    argparse.ArgumentDefaultsHelpFormatter,
    argparse.RawDescriptionHelpFormatter,
):
    """Custom help formatter for argparse to handle default values and description."""


def __check_dir(s: str) -> str:
    if Path(s).is_dir():
        return s
    raise argparse.ArgumentTypeError(f"{s!r} is not a dir.")


def __check_file(s: str) -> str:
    path = Path(s)
    if path.is_file() or not path.exists():
        return s
    if not path.parent.is_dir():
        raise argparse.ArgumentTypeError(f"Base dir of {s!r} does not exist.")
    raise argparse.ArgumentTypeError(f"{s!r} is not a file.")


def __check_positive(s: str) -> int:
    v = int(s)
    if v >= 1:
        return v
    raise argparse.ArgumentTypeError(f"{s!r} is not a positive integer.")


def __parse_args() -> argparse.Namespace:
    usage = textwrap.dedent(
        """
    note:
        This package and tool are mainly made for getjump.
        https://github.com/eggplants/getjump
    """,
    )

    parser = argparse.ArgumentParser(
        prog="mkbook",
        description="Make directory including pictures into a PDF book",
        formatter_class=(
            lambda prog: MBHelpFormatter(
                prog,
                width=shutil.get_terminal_size(fallback=(120, 50)).columns,
                max_help_position=40,
            )
        ),
        epilog=usage,
    )
    parser.add_argument(
        "target_dir",
        type=__check_dir,
        help="target dir",
        metavar="PATH",
    )
    parser.add_argument(
        "save_path",
        type=__check_file,
        help="saved pdf file path",
        metavar="PATH",
    )
    parser.add_argument(
        "-f",
        "--font-size",
        type=__check_positive,
        help="overwrite if pdf path exists",
        default=20,
    )
    parser.add_argument("-F", "--tt-font", type=__check_file, help="truetype font file")
    parser.add_argument("-o", "--overwrite", action="store_true")
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"%(prog)s {__version__}",
    )
    return parser.parse_args()


def main() -> None:
    """Main function to run the command line tool."""
    args = __parse_args()
    status = 0
    save_path = Path(args.save_path)
    if save_path.exists() and not args.overwrite:
        print(
            f"{save_path.resolve()} already exists. Use `-o` to overwrite.",
            file=sys.stderr,
        )
        status = 2
    else:
        font_path = None if args.tt_font == argparse.SUPPRESS else args.tt_font
        m = MakeBook(font_path=font_path)
        m.make(args.save_path, args.target_dir, font_size=args.font_size)
        print(f"Done: {save_path.resolve()}")
    sys.exit(status)


if __name__ == "__main__":
    main()



"""Create a PDF book from images."""

from __future__ import annotations

import importlib.resources
import io
import os
import re
from functools import cmp_to_key
from pathlib import Path
from typing import TYPE_CHECKING, Literal

from PIL import Image as PImage
from reportlab.lib.colors import black
from reportlab.lib.enums import TA_CENTER
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import ParagraphStyle
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import (
    Image,
    PageBreak,
    Paragraph,
    SimpleDocTemplate,
    Spacer,
    Table,
)

if TYPE_CHECKING:
    from reportlab.platypus.flowables import Flowable


class MakeBook:
    """Create a PDF book from images."""

    default_font = Path(str(importlib.resources.files("mkbook"))) / "fonts" / "RampartOne-Regular.ttf"
    img_extensions = PImage.registered_extensions()

    def __init__(self, font_path: str | Path | None = None) -> None:
        """Initialize the MakeBook class.

        Args:
            font_path (str | Path | None): Path to the font file. If None, use the default font.
        """
        self.set_font(font_path)

    def set_font(self, font_path: str | Path | None = None) -> None:
        """Set the font for the PDF.

        Args:
            font_path (str | Path | None): Path to the font file. If None, use the default font.
        """
        font_path = self.default_font if font_path is None else font_path
        font_name = Path(font_path).name.split(".")[0]
        pdfmetrics.registerFont(TTFont(font_name, font_path, asciiReadable=True))
        self.font_path, self.font_name = font_path, font_name

    def make(self, save_path: Path | str, target_path: str, font_size: int = 20) -> None:
        """Create a PDF book from images.

        Args:
            save_path (Path | str): Path to save the PDF file.
            target_path (str): Path to the target directory containing images.
            font_size (int): Font size for the text in the PDF.
        """
        text_style = ParagraphStyle(
            name="Normal",
            fontName=self.font_name,
            alignment=TA_CENTER,
            fontSize=font_size,
            textColor=black,
            wordWrap="CJK",
        )
        door_text_style = ParagraphStyle(
            name="Normal",
            fontName=self.font_name,
            alignment=TA_CENTER,
            fontSize=font_size * 3,
            textColor=black,
            wordWrap="CJK",
        )
        br = "<br />\n<br />\n"
        stories: list[Literal[0, 1] | Flowable] = [
            0,
            Paragraph(Path(target_path).name, door_text_style),
            PageBreak(),
        ]
        sizes = [A4, A4]
        cnt = 1
        tree = list(os.walk(target_path))
        tree = self.sort_v(tree)
        tree_size = len(tree)
        for idx, (root, _dirs, files) in enumerate(tree):
            print(root)
            root_filename = Path(root).name
            files = sorted(f for f in files if Path(f).suffix in self.img_extensions)  # noqa: PLW2901
            w, h = self.get_size(root, files)
            sizes.append((w, h))
            if len(files) > 0:
                stories.extend(
                    (
                        0,
                        Paragraph(
                            root_filename.replace("　", br * 2).replace("]", "]" + br * 2),
                            door_text_style,
                        ),
                        1,
                        Paragraph(f"{cnt}", text_style),
                        PageBreak(),
                    ),
                )
                cnt += 1
            files_size = len(files)
            f_idx = 0
            for f_idx, img in enumerate(files):
                print(
                    f"\033[2K{idx + 1}/{tree_size} ({f_idx + 1}/{files_size})",
                    end="\n\033[A",
                )
                border_img = [[Image(Path(root) / img)]]
                stories.append(
                    Paragraph(root_filename, text_style),
                )
                stories.append(Spacer(1, font_size))
                stories.append(Table(border_img, w, h))
                stories.append(Spacer(1, font_size))
                stories.append(Paragraph(f"{cnt}", text_style))
                stories.append(PageBreak())
                cnt += 1
            print(f"\033[2K{idx + 1}/{tree_size} ({f_idx}/{files_size})", end="\n\033[A")
        print("\033[2KSaving...")

        (_min_w, _min_h), *_, (max_w, max_h) = sorted(sizes)
        max_w += 400
        max_h += 400
        doc = SimpleDocTemplate(str(save_path), pagesize=(max_w, max_h))

        doc.build(
            [
                Spacer(1, max_h / 2) if s == 0 else Spacer(1, max_h / 8 + font_size * 9) if s == 1 else s
                for s in stories
            ],
        )

    @staticmethod
    def pil_to_bytes(img: PImage.Image) -> bytes:
        """Convert a PIL image to bytes.

        Args:
            img (PIL.Image.Image): The PIL image to convert.

        Returns:
            bytes: The image in bytes format.
        """
        img_bytes = io.BytesIO()
        img.save(img_bytes, format="PNG")
        return img_bytes.getvalue()

    def get_size(self, root: str, files: list[str]) -> tuple[float, float]:
        """Get the size of the largest image in the directory.

        Args:
            root (str): The root directory.
            files (list[str]): List of image files.

        Returns:
            tuple[float, float]: The width and height of the largest image.
        """
        if len(files) == 0:
            return A4
        w, h = sorted(PImage.open(Path(root) / f).size for f in files)[-1]
        return float(w), float(h)

    @staticmethod
    def sort_v(tree: list[tuple[str, list[str], list[str]]]) -> list[tuple[str, list[str], list[str]]]:
        """Sort the directory tree.

        Args:
            tree (list[tuple[str, list[str], list[str]]]): The directory tree.

        Returns:
            list[tuple[str, list[str], list[str]]]: The sorted directory tree.
        """

        def cmp(a: tuple[str, list[str], list[str]], b: tuple[str, list[str], list[str]]) -> int:
            def norm(s: str) -> str:
                tr = str.maketrans("１２３４５６７８９０", "1234567890")  # noqa: RUF001
                s = s.translate(tr)
                return re.sub(r"(\d+)", lambda m: m.group(1).zfill(30), s)

            sa, sb = norm(a[0]), norm(b[0])
            return -1 if sa < sb else 1 if sa > sb else 0

        return sorted(tree, key=cmp_to_key(cmp))



""".. include:: ../README.md"""  # noqa: D415

import importlib.metadata

try:
    __version__ = importlib.metadata.version(__name__)
except importlib.metadata.PackageNotFoundError:
    __version__ = "0.0.0"

from .mkbook import MakeBook

__all__ = ["MakeBook"]
